==> retro_data_structures-0.26.0-py3-none-any.whl: retro_data_structures/properties/corruption/archetypes/DynamicLightSpotlight.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection
from retro_data_structures.properties.corruption.core.Spline import Spline

if typing.TYPE_CHECKING:
    class DynamicLightSpotlightJson(typing_extensions.TypedDict):
        spotlight_angle: json_util.JsonObject
        spotlight_angle_duration: float
        spotlight_angle_loops: bool
    

@dataclasses.dataclass()
class DynamicLightSpotlight(BaseProperty):
    spotlight_angle: Spline = dataclasses.field(default_factory=Spline, metadata={
        'reflection': FieldReflection[Spline](
            Spline, id=0xfeeca0d6, original_name='SpotlightAngle', from_json=Spline.from_json, to_json=Spline.to_json
        ),
    })
    spotlight_angle_duration: float = dataclasses.field(default=0.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x79aa798b, original_name='SpotlightAngleDuration'
        ),
    })
    spotlight_angle_loops: bool = dataclasses.field(default=False, metadata={
        'reflection': FieldReflection[bool](
            bool, id=0x2fd08300, original_name='SpotlightAngleLoops'
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.CORRUPTION

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 3:
            return None
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0xfeeca0d6
        spotlight_angle = Spline.from_stream(data, property_size)
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0x79aa798b
        spotlight_angle_duration = struct.unpack('>f', data.read(4))[0]
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0x2fd08300
        spotlight_angle_loops = struct.unpack('>?', data.read(1))[0]
    
        return cls(spotlight_angle, spotlight_angle_duration, spotlight_angle_loops)

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x03')  # 3 properties

        data.write(b'\xfe\xec\xa0\xd6')  # 0xfeeca0d6
        before = data.tell()
        data.write(b'\x00\x00')  # size placeholder
        self.spotlight_angle.to_stream(data)
        after = data.tell()
        data.seek(before)
        data.write(struct.pack(">H", after - before - 2))
        data.seek(after)

        data.write(b'y\xaay\x8b')  # 0x79aa798b
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.spotlight_angle_duration))

        data.write(b'/\xd0\x83\x00')  # 0x2fd08300
        data.write(b'\x00\x01')  # size
        data.write(struct.pack('>?', self.spotlight_angle_loops))

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("DynamicLightSpotlightJson", data)
        return cls(
            spotlight_angle=Spline.from_json(json_data['spotlight_angle']),
            spotlight_angle_duration=json_data['spotlight_angle_duration'],
            spotlight_angle_loops=json_data['spotlight_angle_loops'],
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'spotlight_angle': self.spotlight_angle.to_json(),
            'spotlight_angle_duration': self.spotlight_angle_duration,
            'spotlight_angle_loops': self.spotlight_angle_loops,
        }


def _decode_spotlight_angle_duration(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>f', data.read(4))[0]


def _decode_spotlight_angle_loops(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>?', data.read(1))[0]


_property_decoder: typing.Dict[int, typing.Tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0xfeeca0d6: ('spotlight_angle', Spline.from_stream),
    0x79aa798b: ('spotlight_angle_duration', _decode_spotlight_angle_duration),
    0x2fd08300: ('spotlight_angle_loops', _decode_spotlight_angle_loops),
}

==> retro_data_structures-0.29.0-py3-none-any.whl: retro_data_structures/properties/corruption/archetypes/DynamicLightSpotlight.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection
from retro_data_structures.properties.corruption.core.Spline import Spline

if typing.TYPE_CHECKING:
    class DynamicLightSpotlightJson(typing_extensions.TypedDict):
        spotlight_angle: json_util.JsonObject
        spotlight_angle_duration: float
        spotlight_angle_loops: bool
    

@dataclasses.dataclass()
class DynamicLightSpotlight(BaseProperty):
    spotlight_angle: Spline = dataclasses.field(default_factory=Spline, metadata={
        'reflection': FieldReflection[Spline](
            Spline, id=0xfeeca0d6, original_name='SpotlightAngle', from_json=Spline.from_json, to_json=Spline.to_json
        ),
    })
    spotlight_angle_duration: float = dataclasses.field(default=0.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x79aa798b, original_name='SpotlightAngleDuration'
        ),
    })
    spotlight_angle_loops: bool = dataclasses.field(default=False, metadata={
        'reflection': FieldReflection[bool](
            bool, id=0x2fd08300, original_name='SpotlightAngleLoops'
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.CORRUPTION

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 3:
            return None
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0xfeeca0d6
        spotlight_angle = Spline.from_stream(data, property_size)
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0x79aa798b
        spotlight_angle_duration = struct.unpack('>f', data.read(4))[0]
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0x2fd08300
        spotlight_angle_loops = struct.unpack('>?', data.read(1))[0]
    
        return cls(spotlight_angle, spotlight_angle_duration, spotlight_angle_loops)

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x03')  # 3 properties

        data.write(b'\xfe\xec\xa0\xd6')  # 0xfeeca0d6
        before = data.tell()
        data.write(b'\x00\x00')  # size placeholder
        self.spotlight_angle.to_stream(data)
        after = data.tell()
        data.seek(before)
        data.write(struct.pack(">H", after - before - 2))
        data.seek(after)

        data.write(b'y\xaay\x8b')  # 0x79aa798b
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.spotlight_angle_duration))

        data.write(b'/\xd0\x83\x00')  # 0x2fd08300
        data.write(b'\x00\x01')  # size
        data.write(struct.pack('>?', self.spotlight_angle_loops))

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("DynamicLightSpotlightJson", data)
        return cls(
            spotlight_angle=Spline.from_json(json_data['spotlight_angle']),
            spotlight_angle_duration=json_data['spotlight_angle_duration'],
            spotlight_angle_loops=json_data['spotlight_angle_loops'],
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'spotlight_angle': self.spotlight_angle.to_json(),
            'spotlight_angle_duration': self.spotlight_angle_duration,
            'spotlight_angle_loops': self.spotlight_angle_loops,
        }


def _decode_spotlight_angle_duration(data: typing.BinaryIO, property_size: int) -> float:
    return struct.unpack('>f', data.read(4))[0]


def _decode_spotlight_angle_loops(data: typing.BinaryIO, property_size: int) -> bool:
    return struct.unpack('>?', data.read(1))[0]


_property_decoder: dict[int, tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0xfeeca0d6: ('spotlight_angle', Spline.from_stream),
    0x79aa798b: ('spotlight_angle_duration', _decode_spotlight_angle_duration),
    0x2fd08300: ('spotlight_angle_loops', _decode_spotlight_angle_loops),
}
